@@ -582,7 +582,7 @@ class RealTimeSignalEnsemblingParalleLauncher():
         cumulated_signals['ts'] = cumulated_signals['date'].values.astype(np.int64) // 10 ** 9
 
         print('discretizing the signal')
-        discrete_daily_signals = self.minutely_apply_standard_mean_two_states_discretization(signals_df=cumulated_signals, s=self.minutely_ia_lookback_window)
+        discrete_daily_signals = self.minutely_apply_standard_mean_discretization(signals_df=cumulated_signals, s=self.minutely_ia_lookback_window)
         print('discretization done')
         cumulated_signals['trend']=discrete_daily_signals
 
@@ -692,30 +692,9 @@ class RealTimeSignalEnsemblingParalleLauncher():
         np.save(discretized_saving_path, self.day_discretized_signal_np)
 
 
-    def daily_apply_standard_mean_two_states_discretization_last_signal(self):
-        signal_columns = [self.day_signal_mapping[me_sig] for me_sig in self.day_calib_signals_list]
-        prediction_index = self.day_signal_df.shape[0]-1
-        starting_train = prediction_index - self.daily_ia_lookback_window
-        stopping_train = prediction_index
 
-        local_signal_df = self.day_signal_df[signal_columns]
-        full_y_true = self.day_signal_df['close'].pct_change().fillna(0.)
 
-        local_rolling_np = local_signal_df.values[starting_train:stopping_train, :]
-
-        y_train_true = full_y_true[starting_train:stopping_train]
-        y_train_pred = np.mean(local_rolling_np, axis=1)
-        print('finding the best day discretization')
-        print(str(len(y_train_true)))
-        print(str(len(y_train_pred)))
-        splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization(y_train_pred, y_train_true)
-        y_next_pred = np.nanmean(local_signal_df.values[prediction_index, :])
-        print('day continuous prediction ' + str(y_next_pred))
-        y_next_pred_discrete = signal_utility.apply_discretization_two_states(y_next_pred, splitting_threshold)
-        print('day discrete prediction ' + str(y_next_pred_discrete))
-        return y_next_pred_discrete
-
-    def minutely_apply_standard_mean_two_states_discretization_last_signal(self):
+    def minutely_apply_standard_mean_discretization_last_signal(self, method='find_best_three_states_discretization'):
         signal_columns = [self.minute_signal_mapping[me_sig] for me_sig in self.min_calib_signals_list]
         prediction_index = self.minute_signal_df.shape[0]-1
         starting_train = prediction_index - self.minutely_ia_lookback_window
@@ -731,32 +710,80 @@ class RealTimeSignalEnsemblingParalleLauncher():
         print('finding the best minute discretization')
         print(str(len(y_train_true)))
         print(str(len(y_train_pred)))
-        splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization(y_train_pred, y_train_true)
-        y_next_pred = np.nanmean(local_signal_df.values[prediction_index, :])
-        print('minute continuous prediction '+str(y_next_pred))
-        y_next_pred_discrete = signal_utility.apply_discretization_two_states(y_next_pred, splitting_threshold)
+
+        if method == 'find_best_three_states_discretization':
+            lower_quantile_threshold, upper_quantile_threshold, lower_threshold, upper_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_three_states_discretization(
+                y_train_pred, y_train_true)
+            y_next_pred = np.nanmean(local_signal_df.values[prediction_index, :])
+            y_next_pred_discrete = signal_utility.apply_discretization_three_states(y_next_pred, upper_threshold,
+                                                                                    lower_threshold)
+        elif method == 'find_best_three_states_discretization_lo':
+            lower_quantile_threshold, upper_quantile_threshold, lower_threshold, upper_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_three_states_discretization_lo(
+                y_train_pred, y_train_true)
+            y_next_pred = np.nanmean(local_signal_df.values[prediction_index, :])
+            y_next_pred_discrete = signal_utility.apply_discretization_three_states_lo(y_next_pred, upper_threshold,
+                                                                                       lower_threshold)
+        elif method == 'find_best_two_states_discretization':
+            splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization(
+                y_train_pred, y_train_true)
+
+            y_next_pred = np.nanmean(local_signal_df.values[prediction_index, :])
+            y_next_pred_discrete = signal_utility.apply_discretization_two_states(y_next_pred, splitting_threshold)
+        elif method == 'find_best_two_states_discretization_lo':
+            splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization_lo(
+                y_train_pred, y_train_true)
+            y_next_pred = np.nanmean(local_signal_df.values[prediction_index, :])
+            y_next_pred_discrete = signal_utility.apply_discretization_two_states_lo(y_next_pred, splitting_threshold)
+
         print('minute discrete prediction '+str(y_next_pred_discrete))
         return y_next_pred_discrete
 
-    def daily_apply_standard_mean_two_states_discretization(self, signals_df=None, s=21):
-        day_signal_columns = [self.day_signal_mapping[me_sig] for me_sig in self.day_calib_signals_list]
-        signals_np = signals_df[day_signal_columns].values#[:, len(self.generic_columns):]
-        full_y_true = signals_df['close'].pct_change().fillna(0.)
-        y_pred_discretized = np.zeros(full_y_true.shape)
-        for row_index in range(s, signals_np.shape[0]):
-            starting_train = row_index - s
-            stopping_train = row_index
-            prediction = row_index
-            rolling_np = signals_np[starting_train:stopping_train, :]
-            y_train_true = full_y_true[starting_train:stopping_train]
-            y_train_pred = np.mean(rolling_np, axis=1)
+
+
+    def daily_apply_standard_mean_discretization_last_signal(self, method='find_best_three_states_discretization_lo'):
+        signal_columns = [self.day_signal_mapping[me_sig] for me_sig in self.day_calib_signals_list]
+        prediction_index = self.day_signal_df.shape[0] - 1
+        starting_train = prediction_index - self.daily_ia_lookback_window
+        stopping_train = prediction_index
+
+        local_signal_df = self.day_signal_df[signal_columns]
+        full_y_true = self.day_signal_df['close'].pct_change().fillna(0.)
+
+        local_rolling_np = local_signal_df.values[starting_train:stopping_train, :]
+
+        y_train_true = full_y_true[starting_train:stopping_train]
+        y_train_pred = np.mean(local_rolling_np, axis=1)
+        print('finding the best day discretization')
+        print(str(len(y_train_true)))
+        print(str(len(y_train_pred)))
+
+        if method == 'find_best_three_states_discretization':
+            lower_quantile_threshold, upper_quantile_threshold, lower_threshold, upper_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_three_states_discretization(
+                y_train_pred, y_train_true)
+            y_next_pred = np.nanmean(local_signal_df.values[prediction_index, :])
+            y_next_pred_discrete = signal_utility.apply_discretization_three_states(y_next_pred, upper_threshold,
+                                                                                    lower_threshold)
+        elif method == 'find_best_three_states_discretization_lo':
+            lower_quantile_threshold, upper_quantile_threshold, lower_threshold, upper_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_three_states_discretization_lo(
+                y_train_pred, y_train_true)
+            y_next_pred = np.nanmean(local_signal_df.values[prediction_index, :])
+            y_next_pred_discrete = signal_utility.apply_discretization_three_states_lo(y_next_pred, upper_threshold,
+                                                                                       lower_threshold)
+        elif method == 'find_best_two_states_discretization':
             splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization(
                 y_train_pred, y_train_true)
-            y_next_pred = np.nanmean(signals_np[row_index, :])
-            y_next_pred_discrete = signal_utility.apply_discretization_two_states(y_next_pred, splitting_threshold)
-            y_pred_discretized[prediction] = y_next_pred_discrete
-        return y_pred_discretized
 
+            y_next_pred = np.nanmean(local_signal_df.values[prediction_index, :])
+            y_next_pred_discrete = signal_utility.apply_discretization_two_states(y_next_pred, splitting_threshold)
+        elif method == 'find_best_two_states_discretization_lo':
+            splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization_lo(
+                y_train_pred, y_train_true)
+            y_next_pred = np.nanmean(local_signal_df.values[prediction_index, :])
+            y_next_pred_discrete = signal_utility.apply_discretization_two_states_lo(y_next_pred, splitting_threshold)
+
+
+        print('day discrete prediction ' + str(y_next_pred_discrete))
+        return y_next_pred_discrete
 
     def daily_apply_standard_mean_discretization(self, signals_df=None, s=21, method ='find_best_three_states_discretization_lo'):
         day_signal_columns = [self.day_signal_mapping[me_sig] for me_sig in self.day_calib_signals_list]
@@ -793,7 +820,7 @@ class RealTimeSignalEnsemblingParalleLauncher():
         return y_pred_discretized
 
 
-    def minutely_apply_standard_mean_two_states_discretization(self, signals_df=None, s=21, method ='find_best_two_states_discretization'):
+    def minutely_apply_standard_mean_discretization(self, signals_df=None, s=21, method ='find_best_three_states_discretization'):
         minute_signal_columns = [self.minute_signal_mapping[me_sig] for me_sig in self.min_calib_signals_list]
         signals_np = signals_df[minute_signal_columns].values#[:, len(self.generic_columns):]
         full_y_true = signals_df['close'].pct_change().fillna(0.)
@@ -1228,7 +1255,7 @@ class RealTimeSignalEnsemblingParalleLauncher():
         print(f'last daily alphas signal {last_day_signals}')
         print('discretizing the signal')
         if  self.day_signal_np.shape[0]> self.daily_ia_lookback_window + 1:
-            discrete_signals = self.daily_apply_standard_mean_two_states_discretization_last_signal()
+            discrete_signals = self.daily_apply_standard_mean_discretization_last_signal()
         else :
             print('trouble : not enough daily histo for proper quantization')
             return last_day_ts,np.nanmean(last_day_signals)
@@ -1244,7 +1271,7 @@ class RealTimeSignalEnsemblingParalleLauncher():
         print(f'last minute alphas signal {last_minute_signals}')
         print('discretizing the signal')
         if  self.minute_signal_np.shape[0]> self.minutely_ia_lookback_window + 1:
-            discrete_signals = self.minutely_apply_standard_mean_two_states_discretization_last_signal()
+            discrete_signals = self.minutely_apply_standard_mean_discretization_last_signal()
         else :
             print('trouble : not enough minute histo for proper quantization')
             return last_minute_ts, np.nanmean(last_minute_signals)
