@@ -482,7 +482,6 @@ def discretize_three_states(y_pred, upper_quantile_threshold=0.8, lower_quantile
         print(e)
         print("the quantile could not get computed")
 
-
     if np.isnan(upper_threshold):
         upper_threshold = 0.8
     if np.isnan(lower_threshold):
@@ -614,8 +613,7 @@ def find_best_two_states_discretization(y_pred, y, seed = 42, number_per_year =
         trouble = True
         ## we do nothting
         #print(e)
-    if np.isnan(splitting_quantile_threshold):
-        splitting_quantile_threshold = 0.5
+
 
     y_pred_discrete = discretize_two_states(y_pred, splitting_quantile_threshold=splitting_quantile_threshold)
     perf_df = signal_utility.reconstitute_prediction_perf(y_pred=y_pred_discrete, y_true=y,
@@ -645,16 +643,25 @@ def find_best_two_states_discretization_lo(y_pred, y, seed = 42, number_per_year
         return -sharpe_strat
 
     np.random.seed(seed)
-    # Optimize f
-    w__ = minimize(
-        f_to_optimize,
-        w0,
-        method='SLSQP',
-        #constraints=[const_sum],
-        bounds=const_ind
-    ).x
 
-    splitting_quantile_threshold = w__[0]
+    splitting_quantile_threshold = 0.5
+    trouble = False
+    try:
+        # Optimize f
+        w__ = minimize(
+            f_to_optimize,
+            w0,
+            method='SLSQP',
+            # constraints=[const_sum],
+            bounds=const_ind
+        ).x
+        splitting_quantile_threshold = w__[0]
+    except Exception as e:
+        trouble = True
+        ## we do nothting
+        #print(e)
+
+
     y_pred_discrete = discretize_two_states_lo(y_pred, splitting_quantile_threshold=splitting_quantile_threshold)
     perf_df = signal_utility.reconstitute_prediction_perf(y_pred=y_pred_discrete, y_true=y,
                                                           transaction_cost=False,
@@ -739,17 +746,23 @@ def find_best_three_states_discretization_lo(y_pred, y, seed = 42, number_per_ye
         return -sharpe_strat
 
     np.random.seed(seed)
-    # Optimize f
-    w__ = minimize(
-        f_to_optimize,
-        w0,
-        method='SLSQP',
-        #constraints=[const_sum],
-        bounds=const_ind
-    ).x
 
-    lower_quantile_threshold = w__[0]
-    upper_quantile_threshold = w__[1]
+    lower_quantile_threshold = 0.3
+    upper_quantile_threshold = 0.7
+
+    try:
+        w__ = minimize(
+            f_to_optimize,
+            w0,
+            method='SLSQP',
+            # constraints=[const_sum],
+            bounds=const_ind
+        ).x
+        lower_quantile_threshold = w__[0]
+        upper_quantile_threshold = w__[1]
+    except Exception:
+        print('trouble converging')
+
     y_pred_discrete = discretize_three_states_lo(y_pred, upper_quantile_threshold=upper_quantile_threshold,
                                  lower_quantile_threshold=lower_quantile_threshold)
     perf_df = signal_utility.reconstitute_prediction_perf(y_pred=y_pred_discrete, y_true=y,
